@@ -307,11 +307,11 @@ designTree <- function(tree, method = "unrooted", sparse = FALSE, ...) {
 }
 
 
-# splits now work
 designUnrooted <- function(tree, order = NULL) {
   if (inherits(tree, "phylo")) {
-    if (is.rooted(tree))
-      tree <- unroot(tree)
+    if (is.rooted(tree)) tree <- unroot(tree)
+    tree <- reorder(tree, "postorder")
+#    p <- as.matrix(as.splits(tree)[tree$edge[,2]])
     p <- bipartition(tree)
   }
   if (inherits(tree, "splits")) p <- as.matrix(tree)
